/**
 * Copyright (C) 2011-2012 enStratus Networks Inc
 *
 * ====================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

package org.dasein.cloud.jclouds.cloudsigma;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Locale;

import org.dasein.cloud.CloudException;
import org.dasein.cloud.InternalException;
import org.dasein.cloud.ProviderContext;
import org.dasein.cloud.dc.DataCenter;
import org.dasein.cloud.dc.DataCenterServices;
import org.dasein.cloud.dc.Region;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;


public class CSDC implements DataCenterServices {
	
	   private CloudSigma cloudsigma;
	    
	   public CSDC(@Nonnull CloudSigma provider) { cloudsigma = provider; }

    
    @Override
    public @Nullable DataCenter getDataCenter(@Nonnull String dataCenterId) throws InternalException, CloudException {
        for( Region region : listRegions() ) {
            for( DataCenter dc : listDataCenters(region.getProviderRegionId()) ) {
                if( dataCenterId.equals(dc.getProviderDataCenterId()) ) {
                    return dc;
                }
            }
        }
        return null;
    }

    @Override
    public @Nonnull String getProviderTermForDataCenter(@Nonnull Locale locale) {
        return "data center";
    }

    @Override
    public @Nonnull String getProviderTermForRegion(@Nonnull Locale locale) {
        return "region";
    }

    @Override
    public @Nullable Region getRegion(@Nonnull String regionId) throws InternalException, CloudException {
        for( Region region : listRegions() ) {
            if( regionId.equals(region.getProviderRegionId()) ) {
                return region;
            }
        }
        return null;
    }

    @Override
    public @Nonnull Collection<DataCenter> listDataCenters(@Nonnull String regionId) throws InternalException, CloudException {
        Region region = getRegion(regionId);
        
        if( region == null ) {
            throw new CloudException("No such region: " + regionId);
        }
        DataCenter dc = new DataCenter();
        
        if( regionId.equals("eu-ch1") ) {                
            dc.setActive(true);
            dc.setAvailable(true);
            dc.setName("Zurich");
            dc.setProviderDataCenterId(regionId+ "-a");
            dc.setRegionId(regionId);
        }
        else if( regionId.equals("us-nv1") ) {
            dc.setActive(true);
            dc.setAvailable(true);
            dc.setName("Las Vegas");
            dc.setProviderDataCenterId(regionId+ "-a");
            dc.setRegionId(regionId);
        }
        else {
            throw new CloudException("Unknown region: " + regionId);
        }
        return Collections.singletonList(dc);
    }

    static private List<Region> regions = null;
    
    @Override
    public @Nonnull Collection<Region> listRegions() throws InternalException, CloudException {
        if( regions == null ) {
            ProviderContext ctx = cloudsigma.getContext();
            
            if( ctx == null ) {
                throw new CloudException("No context was defined for this request");
            }
        	Region region = new Region();
        	
        	String endpoint = ctx.getEndpoint();
        	 if( endpoint == null || endpoint.trim().equals("") || endpoint.startsWith("https://api.cloudsigma.com") || endpoint.equals("https://api.zrh.cloudsigma.com") ) {
        	     region.setActive(true);
                 region.setAvailable(true);
                 region.setName("Switzerland 1");
                 region.setProviderRegionId("eu-ch1");
             }
             else{
        	     region.setActive(true);
                 region.setAvailable(true);
                 region.setName("Nevada 1");
                 region.setProviderRegionId("us-nv1");
             }
        	 regions = Collections.unmodifiableList(Collections.singletonList(region));
        }
        return regions;
    }
}
